/**
	File I/O Plugin
    Copyright (C) 2012 motral.  All rights reserved.
**/

/* Standard C headers */
#include <stdio.h>
#include <stdio.h>
#include <time.h>
#include <syslog.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <glob.h>
     
/* HP webOS PDK headers */
#include "SDL.h"
#include "PDL.h"

static int RunCommandLineTests(void)
{
    printf("File plugin, version 1.0");
    
    // even if we didn't process any parameters, we'll exit early.
    return 1;
}

// log(filename, textline);
static PDL_bool log(PDL_JSParameters* params)
{
    if (PDL_GetNumJSParams(params) != 2) {
        syslog(LOG_INFO, "**** wrong number of parameters for writeLine");
        PDL_JSException(params, "wrong number of parameters for writeLine");
        return PDL_FALSE;
    }

    /* parameters are filename, textline */
    const char* filename = PDL_GetJSParamString(params, 0);
    const char* textline = PDL_GetJSParamString(params, 1);
	
	/* call our output function */
	FILE* f = fopen(filename, "a"); // append mode
	time_t t = time(0);
	struct tm* now = localtime(&t);

	fprintf(f, "[%d-%02d-%02d %02d:%02d.%02d] %s\n", (now->tm_year + 1900), (now->tm_mon + 1), now->tm_mday,
		now->tm_hour, now->tm_min, now->tm_sec, textline);
	fclose(f);

    // send data back to the JavaScript side
    syslog(LOG_WARNING, "*** returning results");
    PDL_Err err;
    err = PDL_JSReply(params, "0");
    if (err) {
        syslog(LOG_ERR, "*** PDL_JSReply failed, %s", PDL_GetError());
    }

    syslog(LOG_WARNING, "*** log complete ***");
    
    return PDL_TRUE;
}

// writeln(filename, textline);
static PDL_bool writeln(PDL_JSParameters* params)
{
    if (PDL_GetNumJSParams(params) != 2) {
        syslog(LOG_INFO, "**** wrong number of parameters for writeLine");
        PDL_JSException(params, "wrong number of parameters for writeLine");
        return PDL_FALSE;
    }

    /* parameters are filename, textline */
    const char* filename = PDL_GetJSParamString(params, 0);
    const char* textline = PDL_GetJSParamString(params, 1);
	
	/* call our output function */
	FILE* f = fopen(filename, "a"); // append mode
	fprintf(f, "%s\n", textline);
	fclose(f);

    // send data back to the JavaScript side
    syslog(LOG_WARNING, "*** returning results");
    PDL_Err err;
    err = PDL_JSReply(params, "0");
    if (err) {
        syslog(LOG_ERR, "*** PDL_JSReply failed, %s", PDL_GetError());
    }

    syslog(LOG_WARNING, "*** writeln complete ***");
    
    return PDL_TRUE;
}

static PDL_bool exist(PDL_JSParameters* params)
{
    if (PDL_GetNumJSParams(params) != 1) {
        syslog(LOG_INFO, "**** wrong number of parameters for exist");
        PDL_JSException(params, "wrong number of parameters for exist");
        return PDL_FALSE;
    }

    /* parameters are filename, textline */
    const char* filename = PDL_GetJSParamString(params, 0);

	int res = access(filename, F_OK);
	
	if (res == 0)
		PDL_JSReply(params, "True");
	else 
		PDL_JSReply(params, "False");
    
	return PDL_TRUE;
}

int main(int argc, char** argv)
{
    // Initialize the SDL library with the Video subsystem
    int result = SDL_Init(SDL_INIT_VIDEO);
   
    if ( result != 0 ) {
        printf("Could not init SDL: %s\n", SDL_GetError());
        exit(1);
    }

    PDL_Init(0);

    // look for special -f switch to test getFiles from command line
    if (!PDL_IsPlugin()) {
        RunCommandLineTests();
        return 0;
    }
    
    // register the js callback
    PDL_RegisterJSHandler("log", log);
    PDL_RegisterJSHandler("writeln", writeln);
	PDL_RegisterJSHandler("exist", exist);
    PDL_JSRegistrationComplete();

    // call a "ready" callback to let JavaScript know that we're initialized
    PDL_CallJS("ready", NULL, 0);
    syslog(LOG_INFO, "**** Registered File Plugin ****");

    // Event descriptor
    SDL_Event event;
    do {
        SDL_WaitEvent(&event);
        syslog(LOG_INFO, "**** SDL_WaitEvent returned with event type %d", event.type);        
    } while (event.type != SDL_QUIT);
    // We exit anytime we get a request to quit the app

    // Cleanup
    PDL_Quit();
    SDL_Quit();

    return 0;
}
