/**
A map control which is a wrapper around Yandex map control.

Add Yandex maps script to index.html, like this:

	<head>
		<title>Maps</title>
		<script src="http://api-maps.yandex.ru/2.0/?load=package.full&mode=debug&lang=ru-RU" type="text/javascript"></script>
		...
	</head>

To initialize a Map control:

	{name: "map", kind: "YandexMap", credentials: "my_bing_app_id"}
	
You can get a handle to the actual Yandex map control uisng hasMap(), like this:

	var yandexMap = this.$.map.hasMap();
	
*/
enyo.kind({
    name: "enyo.YandexMap",
    kind: enyo.Control,
    published: {
        /**
        The latitude of the location.
        */
        latitude: 55.76,
        /**
        The longitude of the location.
        */
        longitude: 37.64,
        /**
        The zoom level of the map view.
        */
        zoom: 10,
        /**
        Show a pin at the center of the current map view.
        */
        showPin: false,
        shownTraffic: false,
        shownAccidents: false,
        /**
        The map type of the map view.  Valid map types are
        'yandex#map' -   "";
        'yandex#satellite' -   "";
        'yandex#hybrid' -   "";
        'yandex#publicMap' -   " ";
        'yandex#publicMapHybrid' -   " ".
        */
        mapType: "yandex#map",
        activeLayer: "yandex",
        /**
        The Yandex Maps Key used to authenticate the application.
        */
        credentials: "",
        /**
        Represents Yandex options to customize the map that is displayed.  Can only be set at create time.  For example,
        {kind: "enyo.Map", options: {showDashboard: false, showCopyright: false}}
        */
        options: ""
    },
    components: [
        {
            name: "connStatus", kind: "PalmService", service: "palm://com.palm.connectionmanager/",
            method: "getStatus", onSuccess: "statusFinished", onFailure: "statusFail", onResponse: "gotResponse",
            subscribe: true
        },
        { kind: "enyo.LogFile", name: "log" }
    ],

    events: {
        onPinClick: "",
        onMapLoadFailure: "",
        onConnectionFailure: "",
        onConnectionAvail: "",
        onCenterChanged: ""
    },

    className: "enyo-map",
    //* @protected
    traffic: null,
    domAttributes: {
        "enyo-pass-events": true
    },

    statusFinished: function (inSender, inResponse) {
        enyo.log("conn inet status: " + inResponse.isInternetConnectionAvailable);
        if (inResponse.isInternetConnectionAvailable) {
            this.doConnectionAvail();
        }
        else {
            this.doConnectionFailure();
        }
    },
    statusFail: function (inSender, inResponse) {
        console.log("conn failed");
        this.doConnectionFailure();
    },

    rendered: function () {
        console.log("YandexMap.rendered()");
        this.inherited(arguments);
        this.$.connStatus.call({ "subscribe": true });
    },
    destroy: function () {
        this.inherited(arguments);
        if (this.pinClickEvent) {
            //Microsoft.Maps.Events.removeHandler(this.pinClickEvent);
        }
    },
    createMap: function () {

        if (this.latitude == null || this.latitude == undefined)
            this.latitude = 55.76;
        if (this.longitude == null || this.longitude == undefined)
            this.longitude = 37.64;

        console.log("YandexMap.createMap() started");
        this.map = new ymaps.Map("mainView_ymapsdiv", {
            center: [this.latitude, this.longitude],
            zoom: this.zoom,
            type: "",
            behaviors: ["drag", "dblClickZoom"]
        });

        //this.map.onboundschange = enyo.dispatch;

        this.map.events.add('boundschange', enyo.bind(this, "onChangeCenter"));

        this.googleType = new ymaps.MapType("google", [function () {
            return new ymaps.Layer('http://mt0.google.com/vt/lyrs=m@176000000&hl=ru&%c', {
                projection: ymaps.projection.sphericalMercator,
                tileTransparent: true
            })
        } ]);

        this.osmType = new ymaps.MapType("osm", [function () {
            return new ymaps.Layer('http://c.tile.openstreetmap.org/%z/%x/%y.png', {
                projection: ymaps.projection.sphericalMercator,
                tileTransparent: true
            })
        } ]);

        console.log("YandexMap.createMap() finished");
        this.mouseEventsEnabled = false;

        this.changeLayer(this.activeLayer);
        console.log("created with layer:" + this.activeLayer);

        if (this.shownTraffic)
            this.showTraffic();

        //this.runTrackTest();
    },

    runTrackTest: function () {
        window.setTimeout(enyo.bind(this, function () {

            this.gpsPosition(55.76, 37.64, true, true);

            window.setTimeout(enyo.bind(this, function () {
                this.gpsPosition(55.761, 37.641, true, true);

                window.setTimeout(enyo.bind(this, function () {
                    this.gpsPosition(55.762, 37.643, true, true);

                    window.setTimeout(enyo.bind(this, function () {
                        this.gpsPosition(55.763, 37.644, true, true);
                    }), 1000);

                }), 1000);

            }), 1000);
        }), 3000);
    },

    onChangeCenter: function (e) {
        if (e.get('newCenter') != null) {
            this.doCenterChanged(e.get('newCenter'));
        }
    },

    changeLayer: function (layerSid) {

        if (layerSid == "yandex")
            this.activeLayer = "yandex#map";
        else
            this.activeLayer = layerSid;

        if (this.map == null)
            return;

        switch (this.activeLayer) {
            case "google":
                //                if (this.googleLayer == null) {
                //                    this.googleLayer = new ymaps.Layer('http://mt0.google.com/vt/lyrs=m@176000000&hl=ru&%c', {
                //                        projection: ymaps.projection.sphericalMercator,
                //                        tileTransparent: true
                //                    });
                //                }
                //                this.map.layers.add(this.googleLayer);
                this.map.setType(this.googleType);
                break;
            case "osm":
                //                if (this.osmLayer == null) {
                //                    // http://c.tile.openstreetmap.org/%z/%x/%y.png - standart
                //                    // http://b.tile.opencyclemap.org/cycle/10/618/320.png - for bycycles
                //                    // http://b.tile2.opencyclemap.org/transport/10/618/320.png - for cars
                //                    // http://otile%d.mqcdn.com/tiles/1.0.0/osm/%z/%x/%y.png - MAPQUEST OPEN
                //                    this.osmLayer = new ymaps.Layer('http://c.tile.openstreetmap.org/%z/%x/%y.png', {
                //                        projection: ymaps.projection.sphericalMercator,
                //                        tileTransparent: true
                //                    });
                //                }
                //                this.map.layers.add(this.osmLayer);
                this.map.setType(this.osmType);
                break;
            default:
                // http://vec%d.maps.yandex.net/tiles?l=map&v=2.28.0&%c&%l
                //                if (this.yandexLayer == null) {
                //                    this.yandexLayer = new ymaps.Layer('http://vec%d.maps.yandex.net/tiles?l=map&v=2.28.0&%c&%l', {
                //                        projection: ymaps.projection.sphericalMercator,
                //                        tileTransparent: true
                //                    });
                //                }
                //                this.map.layers.add(this.yandexLayer);
                this.map.setType("yandex#map");
                break;
        };
    },
    destroyMap: function () {
        this.map = null;
    },
    renderMap: function () {
        this.destroyMap();
        try {
            this.createMap();
            return true;
        } catch (e) {
            enyo.log("create map failed: " + e);
            this.doMapLoadFailure(e);
        }
    },
    showMap: function () {
        this.destroyMap();
        try {
            this.createMap();
            return true;
        } catch (e) {
            enyo.log("create map failed: " + e);
            this.doMapLoadFailure(e);
        }
    },
    //* @public
    /**
    Returns the actual Yandex map control.
    */
    hasMap: function () {
        return (this.map != null) && (this.map != 'undefined');
    },
    /**
    Removes all entities from the map except the dropped pin and the pins in inExcludes.
    */
    clearAll: function (inExcludes) {
        this.map.entities.clear();
        if (this.showPin && this.pin) {
            this.map.entities.push(this.pin);
        }
        if (inExcludes) {
            for (var i = 0, ex; ex = inExcludes[i]; i++) {
                if (ex) {
                    this.map.entities.push(ex);
                }
            }
        }
    },
    /**
    Sets the location of the center of the map view.
    @param {number} inLatitude The latitude of the location.
    @param {number} inLongitude The longitude of the location.
    */
    setCenter: function (inLatitude, inLongitude) {
        if (!isNaN(inLatitude) && !isNaN(inLongitude)) {
            this.latitude = inLatitude;
            this.longitude = inLongitude;
            this.updateCenter();
        }
    },

    gpsPosition: function (inLatitude, inLongitude, lbs, drawTrack) {

        console.log("goto position");
        if (this.pointer == null) {

            //   layout
            //     
            var pointerLayout = ymaps.templateLayoutFactory.createClass('' +
                '<div id="pointer"><img src="$[properties.href]" ' +
				'width="$[properties.iconSize.0]" height="$[properties.iconSize.1]" style="$[properties.css]position:relative;left:$[properties.iconOffset.0]px;top:$[properties.iconOffset.1]px;"/></div>');
            ymaps.layout.storage.add('my#pointerLayout', pointerLayout);

            //    
            this.pointer = new ymaps.Placemark([inLatitude, inLongitude],
                {
                    href: (lbs ? 'images/user_location_lbs.png' : 'images/user_location_gps.png'),
                    iconSize: (lbs ? [24, 24] : [38, 45]),
                    iconOffset: (lbs ? [-12, -12] : [-19, -22]),
                    css: "-webkit-transform:rotate(0deg);"
                }, {
                    iconLayout: "my#pointerLayout"
                }
            );

            //    
            this.map.geoObjects.add(this.pointer);

            if (drawTrack) {
                this.createTrackStartPoint([inLatitude, inLongitude]);
            }

            // remember last position
            this.startLat = inLatitude;
            this.startLon = inLongitude;
        }
        else {
            this.pointer.geometry.setCoordinates([inLatitude, inLongitude]);
            var degree = this.calculateBearing(inLatitude, inLongitude);

            // TODO: 
            // optimise the map refreshing
            // there is 2 update
            // 1. pointer
            // 2. track
            // needs something like
            // map.stopDrawing();
            // map.startDrawing();

            // apply degree to pointer
            // ymaps GetObject automatically updates HTMLElement on properties update
            this.pointer.properties.set("css", "-webkit-transform:rotate(" + degree + "deg);");

            // remember last position
            // for next iteration
            this.startLat = inLatitude;
            this.startLon = inLongitude;

            //    , 
            //      
            if (drawTrack) {
                if (this.track) {
                    var lastIndex = this.track.geometry.getLength();
                    var lastPoint = this.track.geometry.get(lastIndex - 1);
                    if (lastPoint[0] != inLatitude && lastPoint[1] != inLongitude) {
                        this.track.geometry.insert(lastIndex, [inLatitude, inLongitude]);
                    }
                }
                else {
                    this.createTrackStartPoint([inLatitude, inLongitude]);
                }
            }
            else {
                // remove track if exist
                if (this.track) {
                    this.map.geoObjects.remove(this.track);
                    this.track = null;
                }
            }
        }
    },

    createTrackStartPoint: function (point) {
        if (this.map) {
            console.log('starting draw new track');
            //   
            this.track = new ymaps.Polyline([point], {}, {
                balloonCloseButton: false,
                // -     
                strokeColor: "0000FF55", //
                // -  
                strokeWidth: 5
            }
            );

            //    
            this.map.geoObjects.add(this.track);
        }
    },

    startLat: 0,
    startLon: 0,
    calculateBearing: function (lat, lon) {
        var lat1 = this.toRad(this.startLat),
            lat2 = this.toRad(lat);
        var dLon = this.toRad(lon - this.startLon);

        var y = Math.sin(dLon) * Math.cos(lat2);
        var x = Math.cos(lat1) * Math.sin(lat2) -
                Math.sin(lat1) * Math.cos(lat2) * Math.cos(dLon);
        var brng = Math.atan2(y, x);

        return (this.toDeg(brng) + 360) % 360;
    },

    // Converts numeric degrees to radians
    toRad: function (value) {
        return value * Math.PI / 180;
    },

    // Converts radians degrees to numeric
    toDeg: function (value) {
        return value * 180 / Math.PI;
    },

    gpsPositionHide: function () {
        if (this.pointer != null) {
            this.map.geoObjects.remove(this.pointer);
            this.pointer = null;
        }
    },
    pantoPosition: function () {
        if (this.pointer != null) {
            this.map.panTo(this.pointer.geometry.getCoordinates());
            return true;
        }
        return false;
    },
    pantoPositionQuick: function () {
        if (this.pointer != null) {
            this.map.panTo(this.pointer.geometry.getCoordinates(), { delay: 200 });
            return true;
        }
        return false;
    },

    //* @protected
    latitudeChanged: function () {
        this.latitude = Number(this.latitude);
        this.updateCenter();
    },
    longitudeChanged: function () {
        this.longitude = Number(this.longitude);
        this.updateCenter();
    },
    updateCenter: function () {
        console.log('going to new center: ' + this.latitude + '/' + this.longitude);
        if (this.map != null)
            this.map.panTo([this.latitude, this.longitude], { flying: true });
    },
    zoomChanged: function () {
        if (this.map != null)
            this.map.setZoom(this.zoom);
    },
    getZoom: function () {
        return this.map.getZoom();
    },
    setZoom: function (newZoom) {
        this.zoom = newZoom;
        if (this.map != null)
            this.map.setZoom(newZoom);
    },
    resizeMap: function () {
        if (this.map != null)
            this.map.container.fitToViewport(false);
    },
    showPinChanged: function () {
        if (this.showPin) {
        } else {
        }
    },

    // ================================= TRAFFIC ===============================================================

    showTraffic: function () {
        if (this.map != null) {
            if (this.traffic == null)
                this.traffic = new ymaps.control.TrafficControl({ providerKey: "traffic#actual", shown: true });
            this.map.controls.add(this.traffic);
            this.shownTraffic = true;
        }
    },
    hideTraffic: function () {
        if (this.map != null && this.traffic != null) {
            this.map.controls.remove(this.traffic);
            this.shownTraffic = false;
        }
    },

    showRoadEvents: function () {
        if (this.map != null) {
            if (this.traffic != null && !this.shownAccidents) {
                this.traffic.getProvider().state.set('infoLayerShown', true);
                this.shownAccidents = true;
            }
        }
    },
    hideRoadEvents: function () {
        if (this.map != null) {
            if (this.traffic != null && this.shownAccidents) {
                this.traffic.getProvider().state.set('infoLayerShown', false);
                this.shownAccidents = false;
            }
        }
    },

    // ================================= TRAFFIC ===============================================================

    pinClick: function (e) {
        this.doPinClick(e);
    },
    mapTypeChanged: function () {
        this.map.setType(this.mapType);
    },
    // We are using touch events now so no need to use gestures unless for systems that touch events
    // are not supported.  We do that by checking if mouse events are enabled because if touch events
    // are disabled then mouse events should be enabled.
    gesturestartHandler: function (inSender, e) {
        enyo.log("gesturestartHandler started");
        if (!this.mouseEventsEnabled) {
            return;
        }
        this.previousScale = e.scale;
        //this.map.setOptions({ disableUserInput: true });
    },
    gesturechangeHandler: function (inSender, e) {
        enyo.log("gesturechangeHandler started");
        if (!this.mouseEventsEnabled) {
            return;
        }
        var d = this.previousScale - e.scale;
        enyo.log("gesturechangeHandler prev scale: " + d);
        if (Math.abs(d) > 0.25) {
            var z = this.map.getZoom() + (d > 0 ? -1 : +1);
            enyo.log("gesturechangeHandler getZoom(): " + this.map.getZoom());
            enyo.log("gesturechangeHandler setZoom(): " + z);
            this.map.setZoom(z);
            this.previousScale = e.scale;
        }
    },
    gestureendHandler: function (inSender, e) {
        enyo.log("gestureendHandler started");
        if (!this.mouseEventsEnabled) {
            return;
        }
        //this.map.setOptions({ disableUserInput: false });
    }
});