enyo.kind({
    name: "enyo.SPPGPS",
    kind: enyo.Control,
    components: [
		{ kind: "PalmService", name: "sppSubscription", subscribe: true },
		{ kind: "PalmService", name: "sppService" }
    ],
    events: {
        onDeviceFound: "",
        onChooseDevice: "",
        onDeviceConnected: "",
        onDeviceDisconnected: "",
        onLocationFound: "",
        onSignalLost: ""
    },
    connected: false,
    connecting: false,
    waitingForChoose: false,
    connect: function () {
        if (!this.connected && !this.connecting) {
            this.connecting = true;

            // SPP connection instanceId
            this.instanceId = 0;

            // logInfo method will print logs to device screen
            this.logInfo("starting bluetooth...");

            // Call & subscribe to spp service
            this.$.sppSubscription.call({}, {
                service: "palm://com.palm.bluetooth/spp/",
                method: "subscribenotifications",
                onResponse: "sppNotify",
                onFailure: "sppFailure"
            });

            // Retrieve list of trusted bluetooth devices
            this.$.sppService.call({}, {
                service: "palm://com.palm.bluetooth/gap/",
                method: "gettrusteddevices",
                onResponse: "getDevicesResponse",
                onSuccess: "getDevicesFinished",
                onFailure: "getDevicesFail"
            });
        }
    },
    sppFailure: function (inSender, inResponse) {
        this.logInfo("sppFailure: " + enyo.json.stringify(inResponse));
    },

    getDevicesResponse: function (inSender, inResponse) {

        // If we found trusted device(s)...
        if (inResponse.returnValue) {
            this.logInfo("gettrusteddevices: " + inResponse.returnValue);
            this.trustedDevicesFound(inResponse);
        } else {
            this.trustedDevicesNotFound(inResponse);
            this.connecting = false;
        }

    },
    getDevicesFinished: function (inSender, inResponse) {
        this.logInfo("getDevicesFinished: " + enyo.json.stringify(inResponse));
    },
    getDevicesFail: function (inSender, inResponse) {
        this.logInfo("getDevicesFail: " + enyo.json.stringify(inResponse));
    },
    openReadReady: function (inSender, inResponse) {
        this.logInfo("openSuccess: " + enyo.json.stringify(inResponse));

        // Set interval (2500 ms) to call readPort
        enyo.job("readPort", enyo.bind(this, "readPort"), 2500);
    },
    openReadFail: function (inSender, inResponse) {
        this.logInfo("openReadFail: " + enyo.json.stringify(inResponse));
        this.connecting = false;
        this.connected = false;
        this.doDeviceDisconnected();
    },
    readPort: function (inSender, inResponse) {
        this.logInfo("SPP Read Port:");

        // Read port
        this.$.sppService.call({
            "instanceId": this.instanceId,
            "dataLength": 1024
        },
		{
		    service: "palm://com.palm.service.bluetooth.spp/",
		    method: "read",
		    onSuccess: "readPortSuccess",
		    onFailure: "readPortFailure"
		});
    },
    readPortSuccess: function (inSender, inResponse) {
        this.logInfo("Read Success: " + inResponse.returnValue + "<br /> Data Length: " + inResponse.dataLength);

        /* Get the NMEA text output and parse - see NMEA specs online for more deatails*/
        if (inResponse.returnValue === true) {
            if (typeof inResponse.data !== "undefined") {

                var gpsData = inResponse.data;
                var i = 0;
                var t1 = gpsData.indexOf("$GPGSA");
                var t2 = gpsData.indexOf("\n", t1);
                var gpgsaData = gpsData.slice(t1, t2).split(",");

                t1 = gpsData.indexOf("$GPGGA");
                t2 = gpsData.indexOf("\n", t1);
                var gpggaData = gpsData.slice(t1, t2).split(",");

                t1 = gpsData.indexOf("$GPRMC");
                t2 = gpsData.indexOf("\n", t1);
                var gprmcData = gpsData.slice(t1, t2).split(",");


                var gpsMode = "";
                switch (gpgsaData[2]) {
                    case "1":
                        gpsMode = "No Signal.";
                        break;
                    case "2":
                        gpsMode = "2D";
                        break;
                    case "3":
                        gpsMode = "3D";
                        break;
                    default:
                        gpsMode = "Unk.";
                        break;
                }

                if (typeof gpgsaData[0] !== "undefined") {

                    // Get SV numbers
                    var gpsSatNumbers = "";

                    // Flag to print results
                    var printResults = true;

                    for (i = 3; i < 15; i++) {
                        gpsSatNumbers += gpgsaData[i] + " ";
                        if (gpgsaData[i] === undefined) {
                            printResults = false;
                        }
                    }
                    //format and output to screen
                    var latMin = Math.round(((gpggaData[2].slice(2, gpggaData[2].length - 1)) / 60) * 10000);
                    var tmpLat = gpggaData[2].slice(0, 2) + "." + latMin + gpggaData[3];

                    var longMin = Math.round(((gpggaData[4].slice(3, gpggaData[4].length - 1)) / 60) * 10000);
                    var tmpLong = gpggaData[4].slice(0, 3) + "." + longMin + gpggaData[5];

                    if (printResults) {
                        this.logInfo("Mode: " + gpsMode + ", Sat.List(SV\'s): " + gpsSatNumbers + ", lat: " + tmpLat + ", long: " + tmpLong + ", Alt: " + gpggaData[9] + "Meters, Speed: " + gprmcData[7] + "MPH");
                    }

                    this.doLocationFound(tmpLat, tmpLong);

                } else {
                    this.logInfo("Invalid GPS data. Perhaps connection is lost?");
                    this.doSignalLost();
                }
            } else {
                this.logInfo("Error: GPS data undefined.")
                this.doSignalLost();
            }
        } else {
            this.logInfo("Unable to read from SPP Port. Unknown error.");
            this.doSignalLost();
        }

        this.openReadReady(this, { "returnValue": true });
    },
    readPortFailure: function (inSender, inResponse) {
        this.logInfo("readPortFailure");
    },
    selectServiceResponse: function (inSender, inResponse) {
        this.logInfo("selectServiceResponse - " + enyo.json.stringify(inResponse));
    },
    sppNotify: function (inSender, inResponse) {
        this.logInfo("sppNotify: " + enyo.json.stringify(inResponse));

        // SPP connection instanceId
        this.instanceId = inResponse.instanceId;

        for (var key in inResponse) {
            if (key === "notification") {
                switch (inResponse.notification) {
                    case "notifnservicenames":
                        this.logInfo("SPP service name: " + inResponse.services[0]);

                        // Send select service response - When successful, this is when device flashes "connected to palm gps" message
                        this.$.sppService.call(
							{
							    "instanceId": this.instanceId,
							    "servicename": inResponse.services[0]
							},
							{
							    service: "palm://com.palm.bluetooth/spp/",
							    method: "selectservice",
							    onResponse: "selectServiceResponse"
							}
						 );

                        return;
                        break;

                    case "notifnconnected":
                        this.logInfo("SPP Connected");

                        if (inResponse.error === 0) {

                            this.connecting = false;
                            this.connected = true;
                            this.doDeviceConnected(inResponse);

                            this.$.sppService.call({
                                "instanceId": this.instanceId
                            },
							{
							    service: "palm://com.palm.service.bluetooth.spp/",
							    method: "open",
							    onSuccess: "openReadReady",
							    onFailure: "openReadFail"
							});

                        }
			            else {
			                this.logInfo("Connect to device failed with error: " + inResponse.error);
			                this.connecting = false;
			                this.connected = false;
			                this.doDeviceDisconnected();
                        }
                        return;

                    case "notifndisconnected":
                        this.logInfo("Device has terminated the connection or is out of range...");

                        this.connecting = false;
                        this.connected = false;
                        this.doDeviceDisconnected();
                        break;

                    default:
                        break;
                }
            }
        }

    },
    trustedDevicesFound: function (objData) {

        // Address of our bluetooth device
        this.targetAddress = "";

        // Are there any trusted devices containing 'GPS' || 'gps' with name - Change this based on your device id
        var targetDevice = /Holux/i;

        this.foundDevices = [];

        if (objData.trusteddevices) {
            for (i = 0; i < objData.trusteddevices.length; i++) {

                //assumes "GPS" is within the name of the bluetooth device
                //if (objData.trusteddevices[i].name.search(targetDevice) > -1) {
                this.logInfo("found: " + objData.trusteddevices[i].name + ", address: " + objData.trusteddevices[i].address);
                this.foundDevices.push(objData.trusteddevices[i]);
                //}
            }

            if (this.foundDevices.length > 0) {

                if (this.foundDevices.length == 1) {
                    // notify subscribers
                    this.targetAddress = objData.trusteddevices[0].address;
                    this.doDeviceFound(objData.trusteddevices[i]);
                    this.subscribeDeviceService(this.targetAddress);
                }
                else {
                    this.waitingForChoose = true;
                    this.logInfo("waiting for choosing a gps device...");
                    this.doChooseDevice(this.foundDevices);
                }
            }
        }
    },
    subscribeDeviceService: function (address) {

        this.waitingForChoose = false;
        this.targetAddress = address;

        // If bluetooth target address exists
        if (this.targetAddress !== "") {

            // Connect to our bluetooth device
            this.$.sppService.call({
                "address": this.targetAddress
            }, {
                service: "palm://com.palm.bluetooth/spp/",
                method: "connect"
            });
        }
    },

    trustedDevicesNotFound: function (objData) {
        this.logInfo("No Trusted Bluetooth Devices Found: " + enyo.json.stringify(objData));
    },
    log: function (content) {
        this.logInfo(content);
    },
    logInfo: function (content) {
        /* logInfo(content) - Log & display data on the screen
        * Logs content and displays it in reverse-chronological order
        */

        console.log("gps: " + content);
        this.i++;
    },
    disconnect: function () {
        /* disconnect() - Disconnect SPP Device
        * !!!!Very Important!!!!
        * Disconnect from the SPP device when exiting the application!
        */

        // Clear the interval that requests our GPS data
        enyo.job.stop("readPort");

        // Close our SPP connection
        this.$.sppService.call({
            "instanceId": this.instanceId
        },
		{
		    service: "palm://com.palm.service.bluetooth.spp/",
		    method: "close",
		    onSuccess: "closeSuccess",
		    onFailure: "closeFailure"
		});
    },
    closeFailure: function () {
        this.log("closeFailure");

        // Disconnect our SPP instance
        this.$.sppService.call(
			{
			    "address": this.targetAddress,
			    "instanceId": this.instanceId
			},
			{
			    service: "palm://com.palm.bluetooth/spp/",
			    method: "disconnect",
			    onResponse: "sppDisconnectResponse",
			    onSuccess: "sppDisconnectSuccess",
			    onFailure: "sppDisconnectFailure"
			}
		 );
    },
    closeSuccess: function () {
        this.log("closeSuccess");

        // Disconnect our SPP instance
        this.$.sppService.call(
			{
			    "address": this.targetAddress,
			    "instanceId": this.instanceId
			},
			{
			    service: "palm://com.palm.bluetooth/spp/",
			    method: "disconnect",
			    onResponse: "sppDisconnectResponse",
			    onSuccess: "sppDisconnectSuccess",
			    onFailure: "sppDisconnectFailure"
			}
		 );
    },
    sppDisconnectResponse: function (inSender, inResponse) {
        this.log("sppDisconnectResponse: " + enyo.json.stringify(inResponse));
    },
    sppDisconnectSuccess: function (inSender, inResponse) {
        this.log("sppDisconnectSuccess");
    },
    sppDisconnectFailure: function (inSender, inResponse) {
        this.log("sppDisconnectFailure: " + enyo.json.stringify(inResponse));
    },
    unloadHandler: function () {
        /* unloadHandler() - Disconnect SPP Device
        * Called when application is dismissed/closed
        */
        //make sure to disconnect from the SPP SERVICE!
        console.log("start unloading...");
        this.disconnect();
    }
});
